import logging

logging.basicConfig(level=logging.INFO,
                    datefmt='%Y-%m-%d %H:%M:%S',
                    format='%(asctime)s - %(name)s - $(levelname)s - %(lineno)s - %(module)s - %(message)s')
logging.getLogger('requests').setLevel(logging.WARNING)

logger = logging.getLogger(__name__)



import os
from tools.components.file_utils import search_files, create_product_base_class, create_init_file, create_function_class


def build(product, sdk_root):
    lib_root = os.path.join(os.path.dirname(os.getcwd()), 'lib')
    sdk_root = os.path.join(sdk_root, 'tencentcloud')
    if not os.path.exists(lib_root):
        os.makedirs(lib_root)
        create_init_file(lib_root)
    # 产品对应的sdk路径
    product_sdk_root = os.path.join(sdk_root, product)
    product_lib_root = os.path.join(lib_root, product)
    clients_path = search_files(file_path=product_sdk_root, file_suffix='_client.py', max_depths=3)
    client_infos = []
    all_func_names = []
    same_func_names = []

    for client_path in clients_path:
        # 去掉根路径并去掉.py后拼接包路径
        client = 'tencentcloud%s' % ('.'.join(client_path.split(sdk_root)[1].split(os.sep))).replace('.py', '')
        client_name = client.split('.')[-1]
        client_root = client.replace('.%s' % client_name, '')
        mod = __import__(client, fromlist=[client_name])
        client_class = None
        for x in mod.__dict__.keys():
            if "Client" in x and x != 'AbstractClient':
                client_class = getattr(mod, x)
                break

        if not client_class:
            raise Exception("包中不包含client")
        func_names = [func_name for func_name in list(client_class.__dict__.keys()) if not func_name.startswith('_')]
        docs = {}
        for func_name in func_names:
            func_doc = getattr(client_class, func_name).__doc__.split(':params')
            docs[func_name] = func_doc
            if func_name in all_func_names:
                same_func_names.append(func_name)
            else:
                all_func_names.append(func_name)

        client_infos.append({
            'client_file': clients_path,
            'client_class': client_class.__name__,
            'version': client_root.split('.')[-1],
            'client_root': client_root,
            'func_names': func_names,
            'docs': docs
        })

    # 生成产品根目录
    product_lib_root = os.path.join(lib_root, product)
    if not os.path.exists(product_lib_root):
        os.makedirs(product_lib_root)
    create_init_file(product_lib_root)

    # 生成基于Base的业务类
    create_product_base_class(product=product, product_root=product_lib_root)

    # 生成产品函数
    create_function_class(product=product, product_root=product_lib_root, clients_info=client_infos)


if __name__ == '__main__':
    build(product='tke', sdk_root='E:\\Code\\tencentcloud-sdk-python')



import importlib
import os.path

from tools.components.string_utils import camel_to_snake
from tools.config.logger import logger


def search_files(file_path, file_suffix, max_depths=1):
    """
    :param file_path:
    :param file_suffix:
    :param max_depths:
    :return:
    """
    if not os.path.isdir(file_path) or max_depths == 0:
        return []
    result = []
    cur_depths_objs = [os.path.join(file_path, p) for p in os.listdir(file_path)]
    for obj in cur_depths_objs:
        if os.path.isdir(obj):
            result.extend(search_files(file_path=obj, file_suffix=file_suffix, max_depths=max_depths - 1))
        else:
            if obj.endswith(file_suffix):
                result.append(obj)
                logger.info("查到指定后缀文件:%s" % obj)
    return result


def create_init_file(file_path):
    """
    :param file_path:
    :return:
    """
    if not os.path.exists(file_path):
        os.makedirs(file_path)
    init_file = os.path.join(file_path, '__init__.py')
    if os.path.exists(init_file):
        os.remove(init_file)
        logger.info("删除已存在的__init__.py:%s" % init_file)
    with open(init_file, 'w', encoding='utf-8') as f:
        f.write("# -*- coding:utf-8 -*-\n")
        f.write("\"\"\"\n")
        f.write("自动生成的文件\n")
        f.write("\"\"\"\n")
        logger.info("创建__init__.py文件成功:%s" % init_file)


def create_product_base_class(product, product_root):
    """
    :param product:
    :param product_root:
    :return:
    """
    data = f"""\"\"\"{product}
\"\"\"
from lib.{product}.{product}_base import {product.title()}Base


class {product.title()}({product.title()}Base):
    \"\"\"{product}业务类\"\"\"
    pass
    
    
{product.upper()} = {product.title()}()
"""
    base_file_path = os.path.join(product_root, '%s.py' % product.lower())
    if os.path.exists(base_file_path):
        os.remove(base_file_path)
        logger.info("删除已存在的class文件:%s" % base_file_path)
    with open(base_file_path, 'w', encoding='utf-8') as f:
        f.write(data)


def create_function_class(product, product_root, clients_info):
    versions = ', '.join(['"%s-%s-%s"' %
                          (x['version'][1:5], x['version'][5:7], x['version'][7:9]) for x in clients_info])

    import_content = f"""\"\"\"{product}基础接口封装。 注:自动生成\"\"\"
from lib.models import Model
from lib.utils import retry_with_conditions
"""

    # class_content
    class_content = f"""
    
class {product.title()}Base(Model):
    def init_client_models(self, **kwargs):
        self.business = "{product}"
        self.versions = [{versions}]
"""

    functions_content = ""
    for item in clients_info:
        client_name = '%s_%s' % (item['client_class'], item['version'])
        import_content += f"""from tencentcloud.{product}.{item['version']}.{product}_client import {item['client_class']} as {client_name}
from tencentcloud.{product}.{item['version']} import models as model_{item['version']}
"""
        class_content += f"""        self.client_{item['version']} = {client_name}(**kwargs)
        self.model_{item['version']} = model_{item['version']}
"""

        for func in item['func_names']:
            request_name = '%sRequest' % func
            mod = importlib.import_module('%s.models' % item['client_root'])
            if not hasattr(mod, request_name):
                logger.error("未找到函数%s的请求参数模型%s,跳过函数的自动生成" % (func, request_name))
                continue
            func_class = getattr(__import__('%s.models' % item['client_root'], fromlist=[request_name]), request_name)
            func_doc = getattr(func_class, '__init__').__doc__
            request_model = getattr(mod, request_name)()
            request_params = [x.lstrip('_') for x in request_model.__dict__.keys()]
            for param in request_params:
                func_doc = func_doc.replace('_%s' % param, camel_to_snake(param))
                func_doc = func_doc.replace(param, camel_to_snake(param))
            func_params = ', '.join(['%s=None' % camel_to_snake(x) for x in request_params])
            func_descriptions = ''.join(item['docs'][func]).split('\n')[0]+ ''.join(func_doc)
            if len(request_params) > 0:
                func_content = f"""
    @retry_with_conditions(3, 10)
    def {camel_to_snake(func)}(self, {func_params}, **kwargs):
        \"\"\"{func_descriptions}
        \"\"\"
        req = self.model_{item['version']}.{request_name}()
"""
            else:
                func_content = f"""
    def {camel_to_snake(func)}(self, **kwargs):
        \"\"\"{''.join(item['docs'][func])}
        \"\"\"
        req = self.model_{item['version']}.{request_name}()
"""
            for param in request_params:
                func_content += f"""        req.{param} = {camel_to_snake(param)}
"""
            func_content += f"""        if kwargs:
            req = self.convert_request(req, **kwargs)
        response = self.client_{item['version']}.{func}(req)
        return response
"""
            functions_content += func_content

    function_file_path = os.path.join(product_root, '%s_base.py' % product.lower())
    if os.path.exists(function_file_path):
        os.remove(function_file_path)
        logger.info("删除已存在的class文件:%s" % function_file_path)
    with open(function_file_path, 'w', encoding='utf-8') as f:
        f.write(import_content)
        f.write(class_content)
        f.write(functions_content)



import time


def retry_with_conditions(max_retries, interval_time):
    """
    :param max_retries:
    :param interval_time:
    :return:
    """

    def decorator(func):
        def wrapper(*args, **kwargs):
            retries = 0
            while retries < max_retries:
                try:
                    return func(*args, **kwargs)
                except Exception as e:
                    is_retry = False
                    for retry_message in []:
                        if retry_message in str(e):
                            is_retry = True
                            retries += 1
                            time.sleep(interval_time)
                            break
                    if not is_retry:
                        raise e

        return wrapper



import json


class Model(object):

    def __init(self, user=None, uri='/v2/index.php', method='POST', token=None, region=None, profile=None,
               sign_method=None):
        """
        :param user:
        :param uri:
        :param method:
        :param token:
        :param region:
        :param profile:
        :param sign_method:
        :return:
        """
        self.user = user
        self.uri = uri
        self.method = method
        self.token = token
        self.region = region
        self.profile = profile
        self.sign_method = sign_method
        self.init_client_models(**{
            'user': user,
            'uri': uri,
            'method': method,
            'token': token,
            'region': region,
            'profile': profile,
            'sign_method': sign_method
        })

    def init_client_models(self, **kwargs):
        """
        初始化客户端模型
        :param kwargs:
        :return:
        """
        pass

    @staticmethod
    def convert_request(req, **kwargs):
        """
        转换请求参数
        :param req: 请求模型
        :param kwargs: 其他参数
        :return: 转换后的请求模型
        """
        if kwargs:
            req_dict = req._serialize()
            req_dict.UPDATE(kwargs)
            req_json = json.dumps(req_dict)
            req.from_json_string(req_json)
        if 'header' in kwargs:
            req._headers = kwargs['header']
        return req



from setuptools import setup, find_packages

setup(
    name="tc-auto",  # 包名（pip install 时用）
    version="0.1.1",  # 版本号
    author="wanderingYee",
    author_email="yehongjiang2012@gmail.com",
    description="tencentcloud api自动化封装",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/wanderingYee/TC-Auto",
    packages=find_packages(),  # 自动发现所有包
    install_requires=[  # 依赖项
        "requests==2.32.4",
    ],
    classifiers=[  # PyPI 分类标签
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",  # Python 版本要求
)
